@@ -28,12 +28,8 @@
     __filter[4] = __active ? (__filter[5] == ':' ? 's' : ':') : '-'
 #define ADBLOCK_FILTER_IS_SET(__filter) \
     (__filter[4] != '-' && __filter[5] != '-')
-#ifdef G_ENABLE_DEBUG
     #define adblock_debug(dmsg, darg1, darg2) \
         do { if (midori_debug ("adblock:match")) g_debug (dmsg, darg1, darg2); } while (0)
-#else
-    #define adblock_debug(dmsg, darg1, darg2) /* nothing */
-#endif
 
 static GHashTable* pattern = NULL;
 static GHashTable* keys = NULL;
@@ -651,7 +647,7 @@ adblock_open_preferences_cb (MidoriExtension* extension)
     adblock_show_preferences_dialog (extension, NULL);
 }
 
-static inline gint
+static gint
 adblock_check_rule (GRegex*      regex,
                     const gchar* patt,
                     const gchar* req_uri,
@@ -674,7 +670,7 @@ adblock_check_rule (GRegex*      regex,
     return TRUE;
 }
 
-static inline gboolean
+static gboolean
 adblock_is_matched_by_pattern (const gchar* req_uri,
                                const gchar* page_uri)
 {
@@ -693,7 +689,7 @@ adblock_is_matched_by_pattern (const gchar* req_uri,
     return FALSE;
 }
 
-static inline gboolean
+static gboolean
 adblock_is_matched_by_key (const gchar* req_uri,
                            const gchar* page_uri)
 {
@@ -877,10 +873,8 @@ adblock_resource_request_starting_cb (WebKitWebView*         web_view,
         }
     }
 
-    #ifdef G_ENABLE_DEBUG
     if (midori_debug ("adblock:time"))
         g_test_timer_start ();
-    #endif
     if (adblock_is_matched (req_uri, page_uri))
     {
         blocked_uris = g_object_get_data (G_OBJECT (web_view), "blocked-uris");
@@ -888,11 +882,8 @@ adblock_resource_request_starting_cb (WebKitWebView*         web_view,
         webkit_network_request_set_uri (request, "about:blank");
         g_object_set_data (G_OBJECT (web_view), "blocked-uris", blocked_uris);
     }
-    #ifdef G_ENABLE_DEBUG
     if (midori_debug ("adblock:time"))
         g_debug ("match: %f%s", g_test_timer_elapsed (), "seconds");
-    #endif
-
 }
 
 static void
@@ -1082,8 +1073,6 @@ adblock_app_add_browser_cb (MidoriApp*       app,
     GtkWidget* statusbar;
     GtkWidget* image;
     GList* children;
-    GtkWidget* view;
-    gint i;
 
     statusbar = katze_object_get_object (browser, "statusbar");
     image = NULL;
@@ -1245,7 +1234,7 @@ adblock_compile_regexp (GString* gpatt,
     }
 }
 
-static inline gchar*
+static gchar*
 adblock_add_url_pattern (gchar* prefix,
                          gchar* type,
                          gchar* line)
@@ -1304,7 +1293,7 @@ adblock_add_url_pattern (gchar* prefix,
     return g_string_free (format_patt, should_free);
 }
 
-static inline void
+static void
 adblock_frame_add (gchar* line)
 {
     const gchar* separator = " , ";
@@ -1338,7 +1327,7 @@ adblock_update_css_hash (gchar* domain,
         g_hash_table_insert (blockcssprivate, g_strdup (domain), g_strdup (value));
 }
 
-static inline void
+static void
 adblock_frame_add_private (const gchar* line,
                            const gchar* sep)
 {
@@ -1391,7 +1380,6 @@ adblock_parse_line (gchar* line)
      * Block URL:
      *   http://example.com/ads/banner123.gif
      *   http://example.com/ads/banner*.gif
-     *   http://example.com/ads/*
      * Partial match for "ad":
      *   *ad*
      *   ad
@@ -1553,7 +1541,9 @@ adblock_file_is_up_to_date (gchar* path)
 
         for (i = 0; i <= 15; i++)
         {
-            fgets (line, 2000, file);
+            if (!fgets (line, 2000, file))
+                break;
+
             if (strncmp ("! Expires", line, 9) == 0)
             {
                 gchar** parts = g_strsplit (line, " ", 4);
@@ -1723,7 +1713,6 @@ adblock_deactivate_cb (MidoriExtension* extension,
                        MidoriBrowser*   browser)
 {
     GList* children;
-    GtkWidget* view;
     MidoriApp* app = midori_extension_get_app (extension);
     MidoriWebSettings* settings = katze_object_get_object (app, "settings");
 
