@@ -986,7 +986,17 @@ midori_array_to_file (KatzeArray*  array,
     return FALSE;
 }
 
-static void
+/**
+ * katze_item_set_value_from_columne:
+ * @stmt: prepared statement
+ * @column: column to read
+ * @item: #KatzeItem to populate
+ *
+ * Stores the column in the given #KatzeItem.
+ *
+ * Since: 0.2.7
+ **/
+void
 katze_item_set_value_from_column (sqlite3_stmt* stmt,
                                   gint          column,
                                   KatzeItem*    item)
@@ -1069,6 +1079,17 @@ katze_array_from_statement (sqlite3_stmt* stmt)
         item = katze_item_new ();
         for (i = 0; i < cols; i++)
             katze_item_set_value_from_column (stmt, i, item);
+
+        if (KATZE_ITEM_IS_FOLDER (item))
+        {
+            g_object_unref (item);
+
+            item = KATZE_ITEM (katze_array_new (KATZE_TYPE_ITEM));
+
+            for (i = 0; i < cols; i++)
+                katze_item_set_value_from_column (stmt, i, item);
+        }
+
         katze_array_add_item (array, item);
     }
 
@@ -1102,245 +1123,3 @@ katze_array_from_sqlite (sqlite3*     db,
 
     return katze_array_from_statement (stmt);
 }
-
-/**
- * midori_array_query_recursive:
- * @array: the main bookmark array
- * @fields: comma separated list of fields
- * @condition: condition, like "folder = '%q'"
- * @value: a value to be inserted if @condition contains %q
- * @recursive: if %TRUE include children
- *
- * Stores the result in a #KatzeArray.
- *
- * Return value: a #KatzeArray on success, %NULL otherwise
- *
- * Since: 0.4.4
- **/
-KatzeArray*
-midori_array_query_recursive (KatzeArray*  bookmarks,
-                              const gchar* fields,
-                              const gchar* condition,
-                              const gchar* value,
-                              gboolean     recursive)
-{
-    sqlite3* db;
-    gchar* sqlcmd;
-    char* sqlcmd_value;
-    KatzeArray* array;
-    KatzeItem* item;
-    GList* list;
-
-    g_return_val_if_fail (KATZE_IS_ARRAY (bookmarks), NULL);
-    g_return_val_if_fail (fields, NULL);
-    g_return_val_if_fail (condition, NULL);
-    db = g_object_get_data (G_OBJECT (bookmarks), "db");
-    g_return_val_if_fail (db != NULL, NULL);
-
-    sqlcmd = g_strdup_printf ("SELECT %s FROM bookmarks WHERE %s "
-                              "ORDER BY (uri='') ASC, title DESC", fields, condition);
-    if (strstr (condition, "%q"))
-    {
-        sqlcmd_value = sqlite3_mprintf (sqlcmd, value ? value : "");
-        array = katze_array_from_sqlite (db, sqlcmd_value);
-        sqlite3_free (sqlcmd_value);
-    }
-    else
-        array = katze_array_from_sqlite (db, sqlcmd);
-    g_free (sqlcmd);
-
-    if (!recursive)
-        return array;
-
-    KATZE_ARRAY_FOREACH_ITEM_L (item, array, list)
-    {
-        if (KATZE_ITEM_IS_FOLDER (item))
-        {
-            gchar* parentid = g_strdup_printf ("%" G_GINT64_FORMAT,
-                katze_item_get_meta_integer (item, "id"));
-            KatzeArray* subarray = midori_array_query_recursive (bookmarks,
-                fields, "parentid=%q", parentid, TRUE);
-            katze_item_set_name (KATZE_ITEM (subarray), katze_item_get_name (item));
-            katze_array_add_item (array, subarray);
-
-            g_free (parentid);
-        }
-    }
-    g_list_free (list);
-    return array;
-}
-
-/**
- * midori_array_query:
- * @array: the main bookmark array
- * @fields: comma separated list of fields
- * @condition: condition, like "folder = '%q'"
- * @value: a value to be inserted if @condition contains %q
- *
- * Stores the result in a #KatzeArray.
- *
- * Return value: a #KatzeArray on success, %NULL otherwise
- *
- * Since: 0.4.3
- *
- * Deprecated: 0.4.4: Use midori_array_query_recursive() instead.
- **/
-KatzeArray*
-midori_array_query (KatzeArray*  bookmarks,
-                    const gchar* fields,
-                    const gchar* condition,
-                    const gchar* value)
-{
-    return midori_array_query_recursive (bookmarks, fields, condition, value, FALSE);
-}
-
-static gint64
-count_from_sqlite (sqlite3*     db,
-		   const gchar* sqlcmd)
-{
-    gint64 count = -1;
-    sqlite3_stmt* stmt;
-    gint result;
-    
-    result = sqlite3_prepare_v2 (db, sqlcmd, -1, &stmt, NULL);
-    if (result != SQLITE_OK)
-        return -1;
-
-    g_assert (sqlite3_column_count (stmt) == 1);
-    
-    if ((result = sqlite3_step (stmt)) == SQLITE_ROW)
-	count = sqlite3_column_int64(stmt, 0);
-
-    sqlite3_clear_bindings (stmt);
-    sqlite3_reset (stmt);
-
-    return count;
-}
-
-static gint64
-midori_array_count_recursive_by_id (KatzeArray*  bookmarks,
-				    const gchar* condition,
-				    const gchar* value,
-				    gint64       id,
-				    gboolean     recursive)
-{
-    gint64 count = -1;
-    sqlite3* db;
-    gchar* sqlcmd;
-    char* sqlcmd_value;
-    sqlite3_stmt* stmt;
-    gint result;
-    GList* ids;
-    GList* iter_ids;
-
-    g_return_val_if_fail (condition, -1);
-    g_return_val_if_fail (KATZE_IS_ARRAY (bookmarks), -1);
-    db = g_object_get_data (G_OBJECT (bookmarks), "db");
-    g_return_val_if_fail (db != NULL, -1);
-
-    g_assert(!strstr("parentid", condition));
-
-    if (id > 0)
-	sqlcmd = g_strdup_printf ("SELECT COUNT(*) FROM bookmarks "
-				  "WHERE parentid = %" G_GINT64_FORMAT " AND %s",
-				  id,
-				  condition);
-    else
-	sqlcmd = g_strdup_printf ("SELECT COUNT(*) FROM bookmarks "
-				  "WHERE parentid IS NULL AND %s ",
-				  condition);
-
-    if (strstr (condition, "%q"))
-    {
-        sqlcmd_value = sqlite3_mprintf (sqlcmd, value ? value : "");
-        count = count_from_sqlite (db, sqlcmd_value);
-        sqlite3_free (sqlcmd_value);
-    }
-    else
-        count = count_from_sqlite (db, sqlcmd);
-
-    g_free (sqlcmd);
-
-    if (!recursive || (count < 0))
-        return count;
-
-    ids = NULL;
-
-    if (id > 0)
-	sqlcmd_value = sqlite3_mprintf (
-	    "SELECT id FROM bookmarks "
-	    "WHERE parentid = %" G_GINT64_FORMAT " AND uri = ''", id);
-    else
-	sqlcmd_value = sqlite3_mprintf (
-	    "SELECT id FROM bookmarks "
-	    "WHERE parentid IS NULL AND uri = ''");
-
-    if (sqlite3_prepare_v2 (db, sqlcmd_value, -1, &stmt, NULL) == SQLITE_OK)
-    {
-	g_assert (sqlite3_column_count (stmt) == 1);
-    
-	if ((result = sqlite3_step (stmt)) == SQLITE_ROW)
-	{
-	    gint64* pid = g_new (gint64, 1);
-	    
-	    *pid = sqlite3_column_int64(stmt, 0);
-	    ids = g_list_append (ids, pid);
-	}
-	
-	sqlite3_clear_bindings (stmt);
-	sqlite3_reset (stmt);
-    }
-
-    sqlite3_free (sqlcmd_value);
-
-    iter_ids = ids;
-    while (iter_ids)
-    {
-	gint64 sub_count = midori_array_count_recursive_by_id (bookmarks,
-							       condition,
-							       value,
-							       *(gint64*)(iter_ids->data),
-							       recursive);
-	
-	if (sub_count < 0)
-	{
-	    g_list_free_full (ids, g_free);
-	    return -1;
-	}
-	
-	count += sub_count;
-	iter_ids = g_list_next (iter_ids);
-    }
-	
-    g_list_free_full (ids, g_free);
-    return count;
-}
-
-/**
- * midori_array_count_recursive:
- * @array: the main bookmark array
- * @condition: condition, like "folder = '%q'"
- * @value: a value to be inserted if @condition contains %q
- * @recursive: if %TRUE include children
- *
- * Return value: the number of elements on success, -1 otherwise
- *
- * Since: 0.5.2
- **/
-gint64
-midori_array_count_recursive (KatzeArray*  bookmarks,
-			      const gchar* condition,
-                              const gchar* value,
-			      KatzeItem*   folder,
-                              gboolean     recursive)
-{
-    gint64 id = -1;
-
-    g_return_val_if_fail (!folder || KATZE_ITEM_IS_FOLDER (folder), -1);
-    
-    id = folder ? katze_item_get_meta_integer (folder, "id") : 0;
-
-    return midori_array_count_recursive_by_id (bookmarks, condition,
-					       value, id,
-					       recursive);
-}
