@@ -10,7 +10,7 @@
 */
 
 #include "midori-array.h"
-#include "midori-bookmarks.h"
+#include "midori-bookmarks-db.h"
 #include "midori-history.h"
 #include "midori-preferences.h"
 #include "midori-privatedata.h"
@@ -368,12 +368,12 @@ midori_frontend_diagnostic_dialog (MidoriApp*         app,
     gchar* crash_log = g_build_filename (midori_paths_get_runtime_dir (), "gdb.bt", NULL);
     if (g_access (crash_log, F_OK) == 0)
     {
-        GtkWidget* button = gtk_button_new_with_mnemonic (_("Show last crash _log"));
-        g_signal_connect_data (button, "clicked",
+        GtkWidget* log_button = gtk_button_new_with_mnemonic (_("Show last crash _log"));
+        g_signal_connect_data (log_button, "clicked",
             G_CALLBACK (midori_frontend_crash_log_cb), crash_log,
             (GClosureNotify)g_free, 0);
-        gtk_widget_show (button);
-        gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 4);
+        gtk_widget_show (log_button);
+        gtk_box_pack_start (GTK_BOX (box), log_button, FALSE, FALSE, 4);
     }
     else
         g_free (crash_log);
@@ -381,11 +381,11 @@ midori_frontend_diagnostic_dialog (MidoriApp*         app,
     gchar* gdb = g_find_program_in_path ("gdb");
     if (gdb != NULL)
     {
-        GtkWidget* button = gtk_button_new_with_mnemonic (_("Run in _debugger"));
+        GtkWidget* gdb_button = gtk_button_new_with_mnemonic (_("Run in _debugger"));
         g_signal_connect (button, "clicked",
             G_CALLBACK (midori_frontend_debugger_cb), dialog);
-        gtk_widget_show (button);
-        gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 4);
+        gtk_widget_show (gdb_button);
+        gtk_box_pack_start (GTK_BOX (box), gdb_button, FALSE, FALSE, 4);
     }
     gtk_dialog_set_default_response (GTK_DIALOG (dialog),
         load_on_startup == MIDORI_STARTUP_HOMEPAGE
@@ -446,14 +446,14 @@ midori_normal_app_new (const gchar* config,
         /* It makes no sense to show a crash dialog while running */
         if (!diagnostic_dialog)
         {
-            gboolean success = FALSE;
-            if (execute_commands != NULL && midori_app_send_command (app, execute_commands))
-                success = TRUE;
-            if (open_uris != NULL && midori_app_instance_send_uris (app, open_uris))
-                success = TRUE;
-            if (!execute_commands && !open_uris && midori_app_instance_send_new_browser (app))
-                success = TRUE;
-            if (success)
+            if (execute_commands != NULL)
+                midori_app_send_command (app, execute_commands);
+            if (open_uris != NULL)
+                midori_app_instance_send_uris (app, open_uris);
+            if (!execute_commands && !open_uris)
+                midori_app_instance_send_new_browser (app);
+
+            if (g_application_get_is_registered (G_APPLICATION (app)))
                 return NULL;
         }
 
@@ -487,9 +487,9 @@ midori_normal_app_new (const gchar* config,
     }
     g_free (uri);
 
-    KatzeArray* bookmarks;
+    MidoriBookmarksDb* bookmarks;
     gchar* errmsg = NULL;
-    if (!(bookmarks = midori_bookmarks_new (&errmsg)))
+    if (!(bookmarks = midori_bookmarks_db_new (&errmsg)))
     {
         g_string_append_printf (error_messages,
             _("Bookmarks couldn't be loaded: %s\n"), errmsg);
@@ -585,9 +585,10 @@ midori_normal_app_new (const gchar* config,
     g_signal_connect (app, "add-browser",
         G_CALLBACK (midori_app_add_browser_cb), NULL);
 
+    midori_session_persistent_settings (settings, app);
+
     g_idle_add (midori_load_soup_session_full, settings);
     g_idle_add (midori_load_extensions, app);
-    g_idle_add (midori_load_session, session);
     return app;
 }
 
@@ -595,11 +596,11 @@ void
 midori_normal_app_on_quit (MidoriApp* app)
 {
     MidoriWebSettings* settings = katze_object_get_object (app, "settings");
-    KatzeArray* bookmarks = katze_object_get_object (app, "bookmarks");
+    MidoriBookmarksDb* bookmarks = katze_object_get_object (app, "bookmarks");
     KatzeArray* history = katze_object_get_object (app, "history");
 
     g_object_notify (G_OBJECT (settings), "load-on-startup");
-    midori_bookmarks_on_quit (bookmarks);
+    midori_bookmarks_db_on_quit (bookmarks);
     midori_history_on_quit (history, settings);
     midori_private_data_on_quit (settings);
 
