@@ -25,9 +25,6 @@
     #include <libsoup/soup-cache.h>
 
 #include <config.h>
-#if HAVE_LIBNOTIFY
-    #include <libnotify/notify.h>
-#endif
 
 struct _MidoriPreferences
 {
@@ -402,7 +399,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
         gchar* supports_web_gl = sokoke_js_script_eval (js_context,
             "!!window.WebGLRenderingContext", NULL);
         if (g_strcmp0 (supports_web_gl, "true"))
-            gtk_widget_set_sensitive (button, FALSE);
+            gtk_widget_hide (button);
         g_free (supports_web_gl);
     }
     #endif
@@ -523,16 +520,16 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     INDENTED_ADD (label);
     entry = katze_property_proxy (settings, "http-proxy", "address");
     SPANNED_ADD (entry);
-    g_signal_connect (settings, "notify::proxy-type",
-        G_CALLBACK (midori_preferences_notify_proxy_type_cb), entry);
+    g_signal_connect_object (settings, "notify::proxy-type",
+        G_CALLBACK (midori_preferences_notify_proxy_type_cb), entry, 0);
     midori_preferences_notify_proxy_type_cb (settings, NULL, entry);
     label = gtk_label_new (_("Port"));
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
     INDENTED_ADD (label);
     entry = katze_property_proxy (settings, "http-proxy-port", NULL);
     SPANNED_ADD (entry);
-    g_signal_connect (settings, "notify::proxy-type",
-        G_CALLBACK (midori_preferences_notify_proxy_type_cb), entry);
+    g_signal_connect_object (settings, "notify::proxy-type",
+        G_CALLBACK (midori_preferences_notify_proxy_type_cb), entry, 0);
     midori_preferences_notify_proxy_type_cb (settings, NULL, entry);
     INDENTED_ADD (gtk_event_box_new ());
     label = gtk_label_new (NULL);
@@ -550,8 +547,8 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     gtk_label_set_markup (GTK_LABEL (label), proxy_types->str);
     g_string_free (proxy_types, TRUE);
     SPANNED_ADD (label);
-    g_signal_connect (settings, "notify::proxy-type",
-        G_CALLBACK (midori_preferences_notify_proxy_type_cb), label);
+    g_signal_connect_object (settings, "notify::proxy-type",
+        G_CALLBACK (midori_preferences_notify_proxy_type_cb), label, 0);
     midori_preferences_notify_proxy_type_cb (settings, NULL, label);
 #ifndef HAVE_WEBKIT2
     if (soup_session_get_feature (webkit_get_default_session (), SOUP_TYPE_CACHE))
@@ -636,7 +633,6 @@ midori_preferences_add_extension_category (KatzePreferences*  preferences,
                                            MidoriApp*         app)
 {
     KatzeArray* array;
-    gchar* extension_path;
     GtkWidget* scrolled;
     GtkWidget* addon;
     GList* children;
