@@ -316,8 +316,6 @@ midori_load_soup_session_full (gpointer settings)
     SoupCookieJar* jar;
     gchar* config_file;
     SoupSessionFeature* feature;
-    gboolean have_new_cookies;
-    SoupSessionFeature* feature_import;
 
     midori_load_soup_session (settings);
 
@@ -359,10 +357,8 @@ midori_load_extensions (gpointer data)
     MidoriApp* app = MIDORI_APP (data);
     gchar** keys = g_object_get_data (G_OBJECT (app), "extensions");
     KatzeArray* extensions;
-    #ifdef G_ENABLE_DEBUG
     gboolean startup_timer = midori_debug ("startup");
     GTimer* timer = startup_timer ? g_timer_new () : NULL;
-    #endif
 
     /* Load extensions */
     extensions = katze_array_new (MIDORI_TYPE_EXTENSION);
@@ -370,10 +366,8 @@ midori_load_extensions (gpointer data)
     g_object_set (app, "extensions", extensions, NULL);
     midori_extension_load_from_folder (app, keys, TRUE);
 
-    #ifdef G_ENABLE_DEBUG
     if (startup_timer)
         g_debug ("Extensions:\t%f", g_timer_elapsed (timer, NULL));
-    #endif
 
     return FALSE;
 }
@@ -399,38 +393,6 @@ settings_notify_cb (MidoriWebSettings* settings,
     g_free (config_file);
 }
 
-void
-midori_session_persistent_settings (MidoriWebSettings* settings,
-                                    MidoriApp*         app)
-{
-    g_signal_connect_after (settings, "notify", G_CALLBACK (settings_notify_cb), app);
-}
-
-static void
-midori_browser_action_last_session_activate_cb (GtkAction*     action,
-                                                MidoriBrowser* browser)
-{
-    KatzeArray* old_session = katze_array_new (KATZE_TYPE_ITEM);
-    gchar* config_file = midori_paths_get_config_filename_for_reading ("session.old.xbel");
-    GError* error = NULL;
-    if (midori_array_from_file (old_session, config_file, "xbel-tiny", &error))
-    {
-        KatzeItem* item;
-        KATZE_ARRAY_FOREACH_ITEM (item, old_session)
-            midori_browser_add_item (browser, item);
-    }
-    else
-    {
-        sokoke_message_dialog (GTK_MESSAGE_ERROR,
-            _("The session couldn't be loaded: %s\n"), error->message, FALSE);
-        g_error_free (error);
-    }
-    g_free (config_file);
-    gtk_action_set_sensitive (action, FALSE);
-    g_signal_handlers_disconnect_by_func (action,
-        midori_browser_action_last_session_activate_cb, browser);
-}
-
 static void
 midori_session_accel_map_changed_cb (GtkAccelMap*    accel_map,
                                      gchar*          accel_path,
@@ -442,157 +404,12 @@ midori_session_accel_map_changed_cb (GtkAccelMap*    accel_map,
     g_free (config_file);
 }
 
-static guint save_timeout = 0;
-
-static gboolean
-midori_session_save_timeout_cb (KatzeArray* session)
+void
+midori_session_persistent_settings (MidoriWebSettings* settings,
+                                    MidoriApp*         app)
 {
-    gchar* config_file = midori_paths_get_config_filename_for_writing ("session.xbel");
-    GError* error = NULL;
-    if (!midori_array_to_file (session, config_file, "xbel-tiny", &error))
-    {
-        g_warning (_("The session couldn't be saved. %s"), error->message);
-        g_error_free (error);
-    }
-    g_free (config_file);
-
-    save_timeout = 0;
-    return FALSE;
-}
-
-static void
-midori_browser_session_cb (MidoriBrowser* browser,
-                           gpointer       pspec,
-                           KatzeArray*    session)
-{
-    if (!save_timeout)
-        save_timeout = midori_timeout_add_seconds (
-            5, (GSourceFunc)midori_session_save_timeout_cb, session, NULL);
-}
-
-static void
-midori_app_quit_cb (MidoriBrowser* browser,
-                    KatzeArray*    session)
-{
-    midori_session_save_timeout_cb (session);
-}
-
-static void
-midori_browser_weak_notify_cb (MidoriBrowser* browser,
-                               KatzeArray*    session)
-{
-    g_object_disconnect (browser, "any-signal",
-                         G_CALLBACK (midori_browser_session_cb), session, NULL);
-}
-
-gboolean
-midori_load_session (gpointer data)
-{
-    KatzeArray* saved_session = KATZE_ARRAY (data);
-    MidoriBrowser* browser;
-    MidoriApp* app = katze_item_get_parent (KATZE_ITEM (saved_session));
-    MidoriWebSettings* settings = katze_object_get_object (app, "settings");
-    MidoriStartup load_on_startup;
-    gchar* config_file;
-    KatzeArray* session;
-    KatzeItem* item;
-    gint64 current;
-    gchar** open_uris = g_object_get_data (G_OBJECT (app), "open-uris");
-    gchar** execute_commands = g_object_get_data (G_OBJECT (app), "execute-commands");
-    gchar* uri;
-    guint i = 0;
-    #ifdef G_ENABLE_DEBUG
-    gboolean startup_timer = midori_debug ("startup");
-    GTimer* timer = startup_timer ? g_timer_new () : NULL;
-    #endif
-
-    browser = midori_app_create_browser (app);
-    midori_session_persistent_settings (settings, app);
-
-    config_file = midori_paths_get_config_filename_for_reading ("session.old.xbel");
-    if (g_access (config_file, F_OK) == 0)
-    {
-        GtkActionGroup* action_group = midori_browser_get_action_group (browser);
-        GtkAction* action = gtk_action_group_get_action (action_group, "LastSession");
-        g_signal_connect (action, "activate",
-            G_CALLBACK (midori_browser_action_last_session_activate_cb), browser);
-        gtk_action_set_visible (action, TRUE);
-    }
-    midori_app_add_browser (app, browser);
-    gtk_widget_show (GTK_WIDGET (browser));
-
-    katze_assign (config_file, midori_paths_get_config_filename_for_reading ("accels"));
+    g_signal_connect_after (settings, "notify", G_CALLBACK (settings_notify_cb), app);
     g_signal_connect_after (gtk_accel_map_get (), "changed",
         G_CALLBACK (midori_session_accel_map_changed_cb), NULL);
 
-    load_on_startup = (MidoriStartup)g_object_get_data (G_OBJECT (settings), "load-on-startup");
-    if (katze_array_is_empty (saved_session))
-    {
-        item = katze_item_new ();
-        if (open_uris)
-        {
-            uri = sokoke_magic_uri (open_uris[i], TRUE, TRUE);
-            katze_item_set_uri (item, uri);
-            g_free (uri);
-            i++;
-        } else if (load_on_startup == MIDORI_STARTUP_BLANK_PAGE)
-            katze_item_set_uri (item, "about:new");
-        else
-            katze_item_set_uri (item, "about:home");
-        katze_array_add_item (saved_session, item);
-        g_object_unref (item);
-    }
-
-    session = midori_browser_get_proxy_array (browser);
-    KATZE_ARRAY_FOREACH_ITEM (item, saved_session)
-    {
-        katze_item_set_meta_integer (item, "append", 1);
-        katze_item_set_meta_integer (item, "dont-write-history", 1);
-        if (load_on_startup == MIDORI_STARTUP_DELAYED_PAGES
-         || katze_item_get_meta_integer (item, "delay") == MIDORI_DELAY_PENDING_UNDELAY)
-            katze_item_set_meta_integer (item, "delay", MIDORI_DELAY_DELAYED);
-        midori_browser_add_item (browser, item);
-    }
-
-    current = katze_item_get_meta_integer (KATZE_ITEM (saved_session), "current");
-    if (!(item = katze_array_get_nth_item (saved_session, current)))
-    {
-        current = 0;
-        item = katze_array_get_nth_item (saved_session, 0);
-    }
-    midori_browser_set_current_page (browser, current);
-    if (midori_uri_is_blank (katze_item_get_uri (item)))
-        midori_browser_activate_action (browser, "Location");
-
-    /* `i` also used above; in that case we won't re-add the same URLs here */
-    for (; open_uris && open_uris[i]; i++)
-    {
-        uri = sokoke_magic_uri (open_uris[i], TRUE, TRUE);
-        midori_browser_add_uri (browser, uri);
-        g_free (uri);
-    }
-
-    g_object_unref (settings);
-    g_object_unref (saved_session);
-    g_free (config_file);
-
-    g_signal_connect_after (browser, "add-tab",
-        G_CALLBACK (midori_browser_session_cb), session);
-    g_signal_connect_after (browser, "remove-tab",
-        G_CALLBACK (midori_browser_session_cb), session);
-    g_signal_connect (app, "quit",
-        G_CALLBACK (midori_app_quit_cb), session);
-    g_object_weak_ref (G_OBJECT (session),
-        (GWeakNotify)(midori_browser_weak_notify_cb), browser);
-
-    if (execute_commands != NULL)
-        midori_app_send_command (app, execute_commands);
-
-    #ifdef G_ENABLE_DEBUG
-    if (startup_timer)
-        g_debug ("Session setup:\t%f", g_timer_elapsed (timer, NULL));
-    #endif
-
-    return FALSE;
 }
-
