@@ -870,11 +870,9 @@ sokoke_add_quality_value (const gchar *str,
 static gchar *
 sokoke_accept_languages (const gchar* const * lang_names)
 {
-    GArray *langs_garray = NULL;
+    GString* langs = NULL;
     char *cur_lang = NULL;
     char *prev_lang = NULL;
-    char **langs_array;
-    char *langs_str;
     float delta;
     int i, n_lang_names;
 
@@ -883,7 +881,7 @@ sokoke_accept_languages (const gchar* const * lang_names)
     delta = 0.999 / (n_lang_names - 1);
 
     /* Build the array of languages */
-    langs_garray = g_array_new (TRUE, FALSE, sizeof (char*));
+    langs = g_string_sized_new (n_lang_names);
     for (i = 0; lang_names[i] != NULL; i++)
     {
         cur_lang = sokoke_posix_lang_to_rfc2616 (lang_names[i]);
@@ -901,21 +899,17 @@ sokoke_accept_languages (const gchar* const * lang_names)
 
             /* Add the quality value and append it */
             qv_lang = sokoke_add_quality_value (cur_lang, 1 - i * delta);
-            g_array_append_val (langs_garray, qv_lang);
+            if (langs->len > 0)
+                g_string_append_c (langs, ',');
+            g_string_append (langs, qv_lang);
         }
     }
 
     /* Fallback: add "en" if list is empty */
-    if (langs_garray->len == 0)
-    {
-        gchar* fallback = g_strdup ("en");
-        g_array_append_val (langs_garray, fallback);
-    }
+    if (langs->len == 0)
+        g_string_append (langs, "en");
 
-    langs_array = (char **) g_array_free (langs_garray, FALSE);
-    langs_str = g_strjoinv (", ", langs_array);
-
-    return langs_str;
+    return g_string_free (langs, FALSE);
 }
 
 
@@ -1527,7 +1521,7 @@ midori_settings_save_to_file (MidoriWebSettings* settings,
         }
         else if (type == G_TYPE_PARAM_UINT)
         {
-            gint integer;
+            guint integer;
             g_object_get (settings, property, &integer, NULL);
             if (integer != G_PARAM_SPEC_UINT (pspec)->default_value)
                 g_key_file_set_integer (key_file, "settings", property, integer);
@@ -1587,12 +1581,12 @@ midori_settings_save_to_file (MidoriWebSettings* settings,
             KATZE_ARRAY_FOREACH_ITEM (extension, extensions)
                 if (midori_extension_is_active (extension))
                 {
-                    const gchar* filename = g_object_get_data (G_OBJECT (extension), "filename");
-                    g_return_val_if_fail (filename != NULL, FALSE);
-                    if (filename && strchr (filename, '/'))
-                        g_warning ("%s: %s unexpected /", G_STRFUNC, filename);
+                    const gchar* extension_filename = g_object_get_data (G_OBJECT (extension), "filename");
+                    g_return_val_if_fail (extension_filename != NULL, FALSE);
+                    if (extension_filename && strchr (extension_filename, '/'))
+                        g_warning ("%s: %s unexpected /", G_STRFUNC, extension_filename);
                     gchar* key = katze_object_get_string (extension, "key");
-                    gchar* subname = key ? g_strdup_printf ("%s/%s", filename, key) : g_strdup (filename);
+                    gchar* subname = key ? g_strdup_printf ("%s/%s", extension_filename, key) : g_strdup (extension_filename);
                     g_key_file_set_boolean (key_file, "extensions", subname, TRUE);
                     g_free (key);
                     g_free (subname);
